#include "ampparser.h"

#include <memory>
#include <QApplication>
#include <QCommandLineParser>

class AMPParserPrivate
{
public:
    QApplication &application;
    std::unique_ptr<QCommandLineParser> parser;

    AMPParserPrivate(QApplication &currentApplication)
        : application(currentApplication)
        , parser(std::make_unique<QCommandLineParser>())
    {}

private:
    AMPParserPrivate(const AMPParserPrivate &) = delete;
    AMPParserPrivate(AMPParserPrivate &&)      = delete;
    AMPParserPrivate &operator=(const AMPParserPrivate &) = delete;
    AMPParserPrivate &operator=(AMPParserPrivate &&) = delete;
};

AMPParser::AMPParser(QApplication &application)
    : d(new AMPParserPrivate(application))
{}

AMPParser::~AMPParser()
{
    delete d;
}

AMPParser::AMPParseResult AMPParser::parseCommandLine(CommandLineOptions *options, QString *errorMessage)
{
    //TO DO add translations
    const QCommandLineOption helpOption = d->parser->addHelpOption();

    const QCommandLineOption versionOption = d->parser->addVersionOption();

    const QCommandLineOption objectOption(QStringList() << "o"
                                                        << "object",
                                          QObject::tr("Object to be used."),
                                          "obj");

    d->parser->setSingleDashWordOptionMode(QCommandLineParser::ParseAsLongOptions);
    d->parser->addOption(objectOption);

    if (!d->parser->parse(d->application.arguments()))
    {
        *errorMessage = d->parser->errorText();
        return CommandLineError;
    }

    if (d->parser->isSet(versionOption))
    {
        return CommandLineVersionRequested;
    }

    if (d->parser->isSet(helpOption))
    {
        return CommandLineHelpRequested;
    }

    if (d->parser->isSet(objectOption))
    {
        const QString objectName = d->parser->value(objectOption);

        if (objectName.isNull() || objectName.isEmpty())
        {
            *errorMessage = QObject::tr("Bad object name: ") + objectName;
            return CommandLineError;
        }
        options->objectName = objectName;

        return CommandLineObjectRequested;
    }

    return CommandLineOk;
}

void AMPParser::showHelp()
{
    d->parser->showHelp();
}

void AMPParser::showVersion()
{
    d->parser->showVersion();
}
