#include "aptcontrollerwidget.h"
#include "ui_aptcontrollerwidget.h"

#include <QDebug>
#include <QItemSelectionModel>

AptControllerWidget::AptControllerWidget(AptController *controller, QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::AptControllerWidget)
    , m_controller(controller)
    , model(nullptr)
    , proxyModel(nullptr)
{
    ui->setupUi(this);
    connect(ui->aptPackagesTableView->horizontalHeader(),
            &QHeaderView::sortIndicatorChanged,
            this,
            [this](int index, Qt::SortOrder) {
                if (index == 1)
                {
                    proxyModel->setSortRole(Qt::CheckStateRole);
                }
                else
                {
                    proxyModel->setSortRole(Qt::DisplayRole);
                }
            });
}

AptControllerWidget::~AptControllerWidget()
{
    delete ui;
}

void AptControllerWidget::setPackagesData(const std::vector<Package> &packages)
{
    model      = std::make_unique<PackageModel>(0, this);
    proxyModel = std::make_unique<AMPProxyModel>(std::vector<int>{0});

    for (auto pkg : packages)
    {
        QVariant pkgVar;
        pkgVar.setValue(pkg);
        model->appendRow(pkgVar);
    }

    proxyModel->setSourceModel(model.get());
    proxyModel->setFilterKeyColumn(-1);

    proxyModel->setHeaderData(0, Qt::Horizontal, tr("Name"), Qt::DisplayRole);

    ui->aptPackagesTableView->setModel(proxyModel.get());
    ui->aptPackagesTableView->sortByColumn(0, Qt::AscendingOrder);

    auto tableHeader = ui->aptPackagesTableView->horizontalHeader();
    tableHeader->setStretchLastSection(false);
    tableHeader->setSectionResizeMode(tableHeader->logicalIndex(0), QHeaderView::Stretch);

    connect(proxyModel.get(), &QSortFilterProxyModel::dataChanged, this, &AptControllerWidget::itemClicked);
}

void AptControllerWidget::setFilter(const QString text)
{
    proxyModel->setFilterRegExp(QRegExp(text, Qt::CaseInsensitive, QRegExp::FixedString));
}

void AptControllerWidget::itemClicked(const QModelIndex &topLeft,
                                      const QModelIndex &bottomRight,
                                      const QVector<int> &roles)
{
    bool isSingleColumn   = topLeft == bottomRight;
    bool isCheckBoxColumn = topLeft.column() == checkBoxColumn;
    bool isSingleRole     = roles.size() == 1;
    bool isCheckStateRole = roles[0] == Qt::CheckStateRole;
    if (isSingleColumn && isCheckBoxColumn && isSingleRole && isCheckStateRole)
    {
        ui->aptPackagesTableView->selectionModel()->select(topLeft,
                                                           m_controller->onItemClicked(
                                                               proxyModel->getEntity(topLeft).value<Package>())
                                                               ? QItemSelectionModel::Select | QItemSelectionModel::Rows
                                                               : QItemSelectionModel::Deselect
                                                                     | QItemSelectionModel::Rows);
    }
}

void AptControllerWidget::paintRow(int row, QBrush brush)
{
    for (auto col = 0; col < columnCount; col++)
    {
        proxyModel->setData(proxyModel->index(row, col), brush, Qt::BackgroundRole);
    }
}
