#include "controllersstorage.h"
#include "aptcontroller.h"
#include "defaultcontroller.h"
#include "loadcontroller.h"
#include "repocontroller.h"
#include "rpmcontroller.h"

ControllersStorage::ControllersStorage(DataSourceInterface *dataSource)
    : m_controllers{}
{
    std::unique_ptr<ControllerInterface> defaultController(new DefaultController(dataSource));
    std::unique_ptr<ControllerInterface> loadController(new LoadController(dataSource));
    std::unique_ptr<ControllerInterface> rpmController(new RpmController(dataSource));
    std::unique_ptr<ControllerInterface> aptController(new AptController(dataSource));
    std::unique_ptr<ControllerInterface> repoController(new RepoController(dataSource));

    m_controllers.push_back(std::move(defaultController));
    m_controllers.push_back(std::move(loadController));
    m_controllers.push_back(std::move(rpmController));
    m_controllers.push_back(std::move(aptController));
    m_controllers.push_back(std::move(repoController));
}

ControllerInterface *ControllersStorage::getControllerByIface(QString iface)
{
    for (std::unique_ptr<ControllerInterface> &controller : m_controllers)
    {
        if (controller->getInterface() == iface)
        {
            return controller.get();
        }
    }
    return nullptr;
}

ControllerInterface *ControllersStorage::getControllerByPath(QString path)
{
    for (std::unique_ptr<ControllerInterface> &controller : m_controllers)
    {
        if (controller->getPath() == path)
        {
            return controller.get();
        }
    }
    return nullptr;
}
