#include "repocontroller.h"
#include "repobuttonwidget.h"
#include "repocontrollerwidget.h"

#include <vector>
#include <QDebug>
#include <QInputDialog>
#include <QObject>

class RepoControllerPrivate
{
public:
    RepoControllerPrivate()
        : m_currentRepos()
        , m_buttonWidget(nullptr)
        , m_controllerWidget(nullptr)
        , m_selected(false)
    {}

    RepoControllerPrivate(const RepoControllerPrivate &) = delete;
    RepoControllerPrivate(RepoControllerPrivate &&)      = delete;

    RepoControllerPrivate &operator=(const RepoControllerPrivate &) = delete;
    RepoControllerPrivate &operator=(RepoControllerPrivate &&) = delete;

    QStringList m_currentRepos;

    RepoButtonWidget *m_buttonWidget;
    RepoControllerWidget *m_controllerWidget;

    bool m_selected;
};

RepoController::RepoController(DataSourceInterface *dataSource)
    : BaseController(dataSource)
    , d(new RepoControllerPrivate())
{}

RepoController::~RepoController() {}

QString RepoController::getInterface()
{
    return REPO_INTERFACE_NAME;
}

QString RepoController::getPath()
{
    return REPO_OBJECT_PATH;
}

void RepoController::selected()
{
    d->m_buttonWidget = new RepoButtonWidget(this);

    d->m_controllerWidget = new RepoControllerWidget(this);

    QStringList repos;
    auto backendResult = getDataSource()->repoList();
    if (backendResult.has_value())
    {
        repos = backendResult.value();
    }
    else
    {
        qWarning() << "Failed to get repo list from APT";
    }
    disableButtons();

    getWindow()->setButtonWidget(d->m_buttonWidget);
    getWindow()->setControllerWidget(d->m_controllerWidget);
    getWindow()->selectInterface(getInterface());

    std::vector<Repo> modelData;
    QRegularExpression regExp("(rpm|rpm-dir|rpm-src) (?:(\\[\\w*\\]) )?(.*) (\\w*)");
    for (QString repo : repos)
    {
        auto match = regExp.match(repo);
        modelData.push_back(Repo(match.captured(1), match.captured(2), match.captured(3), match.captured(4), repo));
    }

    d->m_controllerWidget->setRepoList(modelData);

    d->m_controllerWidget->setFilter(QString());
    getWindow()->setSearchText(QString());

    d->m_selected = true;
}

void RepoController::deSelected()
{
    d->m_currentRepos.clear();
    d->m_selected = false;
}

void RepoController::onWindowClose() {}

void RepoController::update()
{
    deSelected();
    selected();
}

void RepoController::onFilterTextChanged(const QString text)
{
    d->m_controllerWidget->setFilter(text);
}

void RepoController::onRemoveButtonPressed()
{
    for (auto repo : d->m_currentRepos)
    {
        getDataSource()->repoRemove(repo);
    }
    update();
}

void RepoController::onAddButtonPressed()
{
    bool ok;
    QString repoName = QInputDialog::getText(d->m_controllerWidget,
                                             QObject::tr("Alterator Packages Repo"),
                                             QObject::tr("Enter repository addres or name"),
                                             QLineEdit::Normal,
                                             "p10",
                                             &ok);
    if (ok && !repoName.isEmpty())
    {
        getDataSource()->repoAdd(repoName);
    }
    update();
}

void RepoController::onSelected(std::vector<Repo> repos)
{
    d->m_currentRepos.clear();

    for (auto repo : repos)
    {
        d->m_currentRepos.append(repo.m_full);
    }

    enableButtons();
}

void RepoController::disableButtons()
{
    d->m_buttonWidget->disableButtons();
}

void RepoController::enableButtons()
{
    d->m_buttonWidget->enableButtons();
}
