#include "repocontrollerwidget.h"
#include "repocontroller.h"
#include "ui_repocontrollerwidget.h"

RepoControllerWidget::RepoControllerWidget(RepoController *controller, QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::RepoControllerWidget)
    , m_controller(controller)
    , model(nullptr)
    , proxyModel(nullptr)
    , addSc(new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_N), this))
    , rmSc(new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_D), this))
    , updateSc(new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_R), this))
{
    ui->setupUi(this);

    connect(addSc, &QShortcut::activated, this, [this]() { this->m_controller->onAddButtonPressed(); });
    connect(rmSc, &QShortcut::activated, this, [this]() { this->m_controller->onRemoveButtonPressed(); });
    connect(updateSc, &QShortcut::activated, this, [this]() { this->m_controller->update(); });
}

RepoControllerWidget::~RepoControllerWidget()
{
    delete ui;
}

void RepoControllerWidget::setRepoList(const std::vector<Repo> &repos)
{
    model      = std::make_unique<RepoModel>(0, this);
    proxyModel = std::make_unique<AMPProxyModel>();

    for (auto repo : repos)
    {
        QVariant repoVar;
        repoVar.setValue(repo);
        model->appendRow(repoVar);
    }

    model->setHeaderData(0, Qt::Horizontal, tr("Repo type"), Qt::DisplayRole);
    model->setHeaderData(1, Qt::Horizontal, tr("Branch"), Qt::DisplayRole);
    model->setHeaderData(2, Qt::Horizontal, tr("URL"), Qt::DisplayRole);
    model->setHeaderData(3, Qt::Horizontal, tr("Component"), Qt::DisplayRole);

    proxyModel->setSourceModel(model.get());
    proxyModel->sort(Qt::AscendingOrder);
    proxyModel->setFilterKeyColumn(-1);

    ui->repoTable->setModel(proxyModel.get());
    ui->repoTable->resizeColumnsToContents();
    ui->repoTable->sortByColumn(1, Qt::AscendingOrder);

    connect(ui->repoTable->selectionModel(),
            &QItemSelectionModel::selectionChanged,
            this,
            &RepoControllerWidget::selected);
}

void RepoControllerWidget::setFilter(const QString text)
{
    proxyModel->setFilterRegExp(QRegExp(text, Qt::CaseInsensitive, QRegExp::FixedString));
}

void RepoControllerWidget::selected(const QItemSelection &selected, const QItemSelection &deselected)
{
    auto rows = ui->repoTable->selectionModel()->selectedRows();
    std::vector<Repo> repos;
    for (auto row : rows)
    {
        repos.push_back(proxyModel->getEntity(row).value<Repo>());
    }
    m_controller->onSelected(repos);
}
