#include "rpmcontrollerwidget.h"
#include "ui_rpmcontrollerwidget.h"

RpmControllerWidget::RpmControllerWidget(RpmController *controller, QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::RpmControllerWidget)
    , m_controller(controller)
    , model(nullptr)
    , proxyModel(nullptr)
{
    ui->setupUi(this);
}

RpmControllerWidget::~RpmControllerWidget()
{
    delete ui;
}

void RpmControllerWidget::setPackagesList(const std::vector<Package> &packages)
{
    model      = std::make_unique<PackageModel>(0, this);
    proxyModel = std::make_unique<RpmProxyModel>(std::vector<int>{0, 1, 2, 3});

    for (auto pkg : packages)
    {
        QVariant pkgVar;
        pkgVar.setValue(pkg);
        model->appendRow(pkgVar);
    }

    proxyModel->setSourceModel(model.get());
    proxyModel->sort(Qt::AscendingOrder);
    proxyModel->setFilterKeyColumn(-1);

    proxyModel->setHeaderData(0, Qt::Horizontal, tr("Name"), Qt::DisplayRole);
    proxyModel->setHeaderData(1, Qt::Horizontal, tr("Version"), Qt::DisplayRole);
    proxyModel->setHeaderData(2, Qt::Horizontal, tr("Release"), Qt::DisplayRole);
    proxyModel->setHeaderData(3, Qt::Horizontal, tr("Architecture"), Qt::DisplayRole);

    ui->rpmPackagesTableView->setModel(proxyModel.get());
    ui->rpmPackagesTableView->resizeColumnsToContents();
    ui->rpmPackagesTableView->sortByColumn(0, Qt::AscendingOrder);

    ui->rpmPackagesTableView->setModel(proxyModel.get());
    connect(ui->rpmPackagesTableView, &QTableView::clicked, this, &RpmControllerWidget::itemClicked);
}

void RpmControllerWidget::setFilter(const QString text)
{
    proxyModel->setFilterRegExp(QRegExp(text, Qt::CaseInsensitive, QRegExp::FixedString));
}

void RpmControllerWidget::itemClicked(const QModelIndex &index)
{
    m_controller->onItemClicked(proxyModel->getEntity(index).value<Package>());
}
