#ifndef DBUSDATASOURCE_H
#define DBUSDATASOURCE_H

#include "datasource/datasourceinterface.h"

#include <QDBusConnection>
#include <QDBusInterface>
#include <QDBusReply>
#include <QDebug>
#include <QStringList>

class DBusDataSourcePrivate;

class DBusDataSource : public QObject, public DataSourceInterface
{
    Q_OBJECT
    Q_INTERFACES(DataSourceInterface)
public:
    DBusDataSource(QString serviceName, QString path, QString ifaceName, QString methodName);
    ~DBusDataSource();

    QStringList getObjectsPathsByIface(QString iface) override;

    std::optional<QStringList> rpmList() override;
    std::optional<int> rpmInstall(const QString package) override;
    std::optional<int> rpmRemove(const QString package) override;
    std::optional<QString> rpmInfo(const QString package) override;
    std::optional<QStringList> rpmFiles(const QString package) override;

    std::optional<int> aptInstall(const QString package) override;
    std::optional<int> aptReinstall(const QString package) override;
    std::optional<int> aptRemove(const QString package) override;
    std::optional<int> aptUpdate() override;
    std::optional<std::vector<QString>> aptListAllPackages() override;
    std::optional<QString> aptLastUpdate() override;

    std::optional<int> repoAdd(const QString repo) override;
    std::optional<QStringList> repoList() override;
    std::optional<int> repoRemove(const QString repo) override;

public slots:
    void aptUpdateSlot(QDBusPendingCallWatcher *call);

signals:
    void aptUpdateFinished(QDBusPendingCallWatcher *call) override;

private:
    QStringList objectByPath(QString iface);

    std::optional<QStringList> dbusCallStringList(const QString &path,
                                                  const QString &interface,
                                                  const QString &method,
                                                  const QString &arg = QString());
    std::optional<int> dbusCallInt(const QString &path,
                                   const QString &interface,
                                   const QString &method,
                                   const QString &arg = QString());

    template<typename T, std::enable_if_t<std::is_same<T, int>::value || std::is_same<T, QStringList>::value, bool> = true>
    std::optional<T> dbusCall(const QString &path,
                              const QString &interface,
                              const QString &method,
                              const QString &arg = QString());

private:
    QDBusConnection m_connection;
    QString m_serviceName{};
    QString m_alteratorPath{};
    QString m_alteratorIfaceName{};
    QString m_alteratorMethodName{};

    QString m_aptPath{};
    QString m_rpmPath{};
    QString m_repoPath{};
};

#endif //DBUSDATASOURCE_H
