/***********************************************************************************************************************
**
** Copyright (C) 2023 BaseALT Ltd. <org@basealt.ru>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**
***********************************************************************************************************************/

#include <iostream>
#include <QApplication>
#include <QLibraryInfo>
#include <QTranslator>

#include "ampapp.h"
#include "constants.h"
#include "datasource/dbusdatasource.h"
#include "mainwindow/mainwindow.h"
//#include "version.h"

#define AMC_VERSION_MAJOR 0
#define AMC_VERSION_MINOR 1
#define AMC_VERSION_PATCH 0

QString getVersion()
{
    return QString("%1.%2.%3").arg(AMC_VERSION_MAJOR).arg(AMC_VERSION_MINOR).arg(AMC_VERSION_PATCH);
}

int main(int argc, char **argv)
{
    QApplication app(argc, argv);

    app.setOrganizationName(QCoreApplication::translate("main", "BaseALT"));
    app.setOrganizationDomain("basealt.ru");
    app.setApplicationName("AMP");
    app.setApplicationVersion(getVersion());

    QLocale locale;

    QTranslator translator;
    QString language = locale.system().name().split("_").at(0);
    translator.load("amp_" + language, ":/");
    app.installTranslator(&translator);

    QTranslator qtBaseTranslator;
    if (qtBaseTranslator.load("qtbase_" + QLocale::system().name(),
                              QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
    {
        qDebug() << "qtBaseTranslator ok";
        app.installTranslator(&qtBaseTranslator);
    }

    DBusDataSource source(ALTERATOR_SERVICE_NAME,
                          ALTERATOR_PATH,
                          ALTERATOR_INTERFACE_NAME,
                          ALTERATOR_GETOBJECTS_METHOD_NAME);

    AMPApp ampApp(&app, &source);

    return ampApp.runApp();
}
