#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "controllers/controllerinterface.h"
#include "mainwindow/mainwindowinterface.h"

#include <QAbstractItemModel>
#include <QItemSelectionModel>
#include <QMainWindow>

namespace Ui
{
class MainWindow;
}

class MainWindowPrivate;
class ControllerInterface;

class MainWindow : public QMainWindow, public MainWindowInterface
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

    void setController(ControllerInterface *controller) override;
    ControllerInterface *getController() override;

    void setModel(QAbstractItemModel *model) override;
    void selectInterface(QString iface) override;

    void setButtonWidget(QWidget *widget) override;
    void setControllerWidget(QWidget *widget) override;
    void closeFromButtonWidget() override;

    void setSearchText(const QString text) override;

protected:
    void closeEvent(QCloseEvent *event) override;

signals:
    void managerChanged(const QItemSelection &newSelection, const QItemSelection &previousSelection);

private slots:
    void onManagerChanged(const QItemSelection &newSelection, const QItemSelection &previousSelection);
    void onFilterTextChanged(const QString text);

private:
    std::unique_ptr<MainWindowPrivate> d;

private:
    MainWindow(const MainWindow &) = delete;
    MainWindow(MainWindow &&)      = delete;
    MainWindow &operator=(const MainWindow &) = delete;
    MainWindow &operator=(MainWindow &&) = delete;
};

#endif // MAINWINDOW_H
