#include "packageswidget.h"
#include "ui_packageswidget.h"

#include <QPushButton>
#include <QStandardItem>

PackagesWidget::PackagesWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::PackagesWidget)
    , findShortcut(new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_F), this))
{
    ui->setupUi(this);

    connect(findShortcut, &QShortcut::activated, this, [this]() { ui->searchPackageLineEdit->setFocus(); });
}

PackagesWidget::~PackagesWidget()
{
    delete ui;
}

void PackagesWidget::setModel(QAbstractItemModel *model)
{
    ui->packageManagersListView->setModel(model);
    connect(ui->packageManagersListView->selectionModel(),
            &QItemSelectionModel::selectionChanged,
            this,
            &PackagesWidget::onSelectionChanged);

    connect(ui->searchPackageLineEdit, &QLineEdit::returnPressed, this, &PackagesWidget::onFilterTextChanged);
    connect(ui->searchButton, &QPushButton::clicked, this, [this](bool) { onFilterTextChanged(); });
}

void PackagesWidget::setButtonWidget(QWidget *widget)
{
    clearLayout(ui->buttonsHorizontalLayout);
    ui->buttonsHorizontalLayout->addWidget(widget);
}

void PackagesWidget::setControllerWidget(QWidget *widget)
{
    clearLayout(ui->controllerGridLayout);
    ui->controllerGridLayout->addWidget(widget);
}

void PackagesWidget::setIndex(QModelIndex index)
{
    ui->packageManagersListView->setCurrentIndex(index);
}

void PackagesWidget::clearLayout(QLayout *layout)
{
    QLayoutItem *child;
    while ((child = layout->takeAt(0)) != nullptr)
    {
        delete child->widget();
        delete child;
    }
}

void PackagesWidget::onSelectionChanged(const QItemSelection &newSelection, const QItemSelection &previousSelection)
{
    emit selectionChanged(newSelection, previousSelection);
}

void PackagesWidget::onFilterTextChanged()
{
    emit filterTextChanged(ui->searchPackageLineEdit->text());
}

void PackagesWidget::setSearchText(const QString text)
{
    ui->searchPackageLineEdit->setText(text);
}
